############################################################
# This script accompanies the book "Windows NT Configuration
# and Maintenance" by Matthew M Lavy and Ashley J Meggitt
# (published by O'Reilly, 1999)
# The script is copyright, 1999 Matthew Lavy & Ashley Meggitt
# 
# WARNING: This script is provided in electronic format as a
# convenience for readers of the book it accompanies. It may
# well require site-specific customisation before it works at
# all, and may cause damage to a workstation if run without this
# customisation (please refer to the book for details).
# DO NOT RUN THIS SCRIPT UNLESS YOU HAVE CHECKED THOROUGHLY THAT
# IT IS SAFE TO DO SO. The authors accept no liability for any
# damage caused by running the script.

#MaintainAndConfigure.pm

package MaintainAndConfigure;
require Exporter;
use Win32::Registry;
use Win32;
use Net::SMTP;

@ISA = qw(Exporter);
@EXPORT = qw(
MCWriteLog
MCRaiseError 
MCSetAdminMachine
MCSetAdminEmail
MCWriteRegistry
MCDeleteRegistry
MCInstallAutoLogon
MCRemoveAutoLogon
MCInstallService
MCRemoveService
MCGetIdentity
MCUpdateScripts
MCConfigFromFile
REG_SZ
REG_DWORD
REG_BINARY
);

sub MCWriteLog($)
{
my @t = localtime;
my $time = "@t[3]\/@t[4]\/@t[5]:@t[2]:@t[1]:@t[0]";
open LOGFILE, ">> $LogFileName";
print LOGFILE "${logcount}\.$time - @_[0]\n";
close LOGFILE;
$logcount++;
}

sub MCRaiseError($)
{
MCWriteLog ($_[0]);
my $errormsg = "A script on this workstation has failed.\nPlease see log file for details.";
`net send $admin_machine $errormsg`;
print "$errormsg\n";

# The lines below require the netutil module to be 
# installed for sending SMTP.
# If it is not installed on your system, rem out these lines

my $smtp = Net::SMTP->new($smtp_host);
$smtp->mail($script_address);
$smtp->to($admin_email);
$smtp->data();
$smtp->datasend("Scripting has failed on $ENV{ComputerName}");
$smtp->datasend($errormsg);
$smtp->dataend();
$smtp->quit;
exit 0;
}

sub MCSetAdminMachine($)
{
$AdminMachine = ($_[0]);
MCWriteLog("Script error messages will be sent to $AdminMachine");
}

sub MCSetAdminEmail($)
{
$AdminEmail = ($_[0]);
MCWriteLog("Script errors will be e-mailed to $AdminEmail");
}

sub MCWriteRegistry($$)
{
# Create usable variable names
my $key = $_[0];
my $RegHandle; #declare to force "my"; this is not initialized yet
my @values = @{$_[1]}; # an array of arrayreferences

if(!Win32::Registry::RegOpenKeyEx
(HKEY_LOCAL_MACHINE,$key,NULL,KEY_ALL_ACCESS,$RegHandle))
{
# If opening of registry key is unsuccessful,
# try creating a new key
if(!Win32::Registry::RegCreateKey
(HKEY_LOCAL_MACHINE,$key,$RegHandle))
{
# failing that, generate a fatal error
MCRaiseError("MCWriteRegistry was unable to open or create
the required registry key");
}
MCWriteLog("Registry Key $key not found, so has been created");
}
MCWriteLog("Registry key $key opened successfully");

# If we got this far, the key has been opened successfully.
# Therefore we can continue.
# So write each value pair, aborting mission if we can't

# Each entry in @values is a name, value, type array
foreach $arrayref(@values)
{
#dereference the three-part array and put the values into
#useful variables
my ($name, $value, $type) = @$arrayref;

# write the value. If that fails, CLOSE THE KEY and raise an error
if(!Win32::Registry::RegSetValueEx
($RegHandle, $name,NULL, $type, $value))
{
Win32::Registry::RegCloseKey($RegHandle); 
MCRaiseError("MCWriteRegistry was unable
to write the required registry value");
}
MCWriteLog("Value $name written");
}
Win32::Registry::RegCloseKey($RegHandle);
return 0; #return an errorcode of 0 for success, 
#rather than letting the value of @_ be returned
}

sub MCDeleteRegistry
{
my $key = shift @_;
my $RegHandle; #declare to force "my"; this is not initialized yet
my $value; # declare to force a "my"

if(!Win32::Registry::RegOpenKeyEx(HKEY_LOCAL_MACHINE,$key,
NULL,KEY_ALL_ACCESS,$RegHandle))
{
# if key didn't exist, raise an error
MCRaiseError("MCWriteRegistry was unable to open 
the required registry key");
}
MCWriteLog("Registry key $key opened successfully");

# If we got this far, the key has been opened successfully.
# Therefore we can continue.
# So delete each value:

foreach $value(@_)
{
if(!Win32::Registry::RegDeleteValue($RegHandle,$value))
{
	# uncomment the lines below to make deletion failure a fatal error
# Win32::Registry::RegCloseKey($RegHandle);
# MCRaiseError("MCDeleteRegistry
#was unable to delete the specified key");
}
MCWriteLog("Value $value deleted successfully");
}
Win32::Registry::RegCloseKey($RegHandle);
}


sub MCInstallAutoLogon($$$)
{
# First turn command-line parameters into a useable format
(my $scriptpath, my $username, my $password) = @_;

# Ensure the logon account is enabled
`net user $username /ACTIVE:Yes`;
if($?) #if this is not 0, account cannot be enabled,raise an error
{
MCRaiseError("Unable to activate user account for autologon");
}
else
{
MCWriteLog("User account $username activated");
}

# Now write the relevant registry entries
# These functions need no error checking as this is built in
# to the registry function
$key = 'SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon';
@values=
(
['AutoAdminLogon',1,REG_SZ],
['DefaultUserName',$username,REG_SZ],
['DefaultPassword',$password,REG_SZ],
['DontDisplayLastUserName',0,REG_SZ],
['RunLogonScriptSync',0,REG_DWORD]
);
MCWriteRegistry($key,\@values);
MCDeleteRegistry($key,'DefaultDomainName','LegalNoticeCaption','LegalNoticeText');
    MCWriteRegistry('SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnce',(['AutoInstalledScript',$scriptpath,REG_SZ]));
MCWriteLog("Script $script installed for autologon");
}


sub MCRemoveAutoLogon
{
# first disable the logon account if the parameter is provided
if(@_)
{
`net user $_[0] /ACTIVE:No`;
if($?) # if net command was unsuccessful
{
# If your environment does not require a fatal error here
# rem out the line below, or replace with an
# MCWriteLog call
MCRaiseError("Unable to disable user account");
}
else
{
MCWriteLog("User account $_[0] disabled");
}
}

# now set registry keys to disable autologon
my $key = 'SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon';
my @values=
(
['AutoAdminLogon',0,REG_SZ],
['DefaultUserName','',REG_SZ],
['DefaultPassword','',REG_SZ],
['DontDisplayLastUserName',1,REG_SZ], 
['RunLogonScriptSync',1,REG_DWORD]
);
MCWriteRegistry($key,\@values);

# if params 2 & 3 are provided, write a LegalNotice
if($_[1] && $_[2])
{
my @values=
(
['LegalNoticeCaption',$_[1],REG_SZ],
['LegalNoticeText',$_[2],REG_SZ]
);
MCWriteRegistry($key, \@values);
}
MCWriteLog("Autologon disabled");
}

sub MCInstallService($)
{
# extract the "name" part of the parameter (e.g. strip the path and .pl)
my $script_name = $_[0];
$script_name =~ s/.*\\(\w+)\.pl/$1/;

# use instsrv to install an instance of srvany (with a plausible name)
`instsrv $script_name srvany.exe`;

# now write the registry parameters used by srvany
$key = "SYSTEM\\CurrentControlSet\\Services\\$script_name";
@values =
(
['Application','srvany.exe',REG_SZ],
['AppParameters',$_[0],REG_SZ]
);
MCWriteRegistry($key,\@values);
MCWriteLog("$script_name installed as a service");
}


sub MCRemoveService($)
{
`instsrv @_[0] /delete`; # use instsrv to remove service by name
if($?)
{
# if a fatal error is not appropriate here, use MCWriteLog instead
MCRaiseError("Unable to remove @_[0] service");
}
else
{
MCWriteLog("@_[0] service successfully deleted");
}
}

sub MCGetIdentity
{
# First check to see if a parameter was passed
if($_[0] != null)
{
# it wasn't, so get identity from the workstation
# first, the computername - no error checking here
# as this is NOT going to fail!
my @name = `net config workstation`;
$name[0]=~ s/.*\\\\(.*)/$1/;
my @lines = `ipconfig`;
# now get the ipaddres
foreach my $IP(@lines)
{
chomp($IP);
if ($IP =~ /^\s+IP.*/)
{
$IP =~ s/.*:\s(.*)/$1/;
goto FOUND_IP; # leap out of loop
}
}
# the line below will only execute if no IP address was found
MCRaiseError("Unable to deduce IP address");
# A successful ipconfig lookup - return the Computer Name and 
# IP address
FOUND_IP: return ($name[0], $IP);
}
else # we use the address.mac file to get the information
{
# open the registry key where the Mac address is stored
my $key = 'SOFTWARE\Description\Microsoft\Rpc\UuidTemporaryData';
my $RegHandle;
if(!Win32::Registry::RegOpenKeyEx
(HKEY_LOCAL_MACHINE,$key,NULL,KEY_ALL_ACCESS,$RegHandle))
{
MCRaiseError("MCGetIdentity unable to open Registry
key where MAC address is stored");
}

# retrieve mac address from the key
if(Win32::Registry::RegQueryValueEx
($RegHandle, 'NetworkAddress',NULL, $type, $rawmac))
{
$machinemac = unpack "H12",$rawmac;
$machinemac =~ s/(\w{2})/$1-/g;
chop($machinemac);
}
else
{
Win32::Registry::RegCloseKey($RegHandle);
MCRaiseError("MCGetIdentity unable to retrieve MAC address");
}

#finally, close the key
Win32::Registry::RegCloseKey($RegHandle);


# Open the MAC address lookup file
open MACFILE, $_[0] || MCRaiseError("MCGetIdentity cannot
open lookup file");

# loop through each line, looking for an entry
# corresponding to @machinemac
$match=0;
while (($line = <MACFILE>) && (!$match))
{
chomp($line); 
	    ($computername,$ipnumber,$macaddress) = split(/\s*[,;:]\s*/,$line); 
if($macaddress eq $machinemac)
{
$match = 1;
}
}
close MACFILE;

if($match)
{
MCWriteLog("Entry for MAC address $macaddress found");
return ($computername,$ipnumber);
}
else
{
MCRaiseError("MCGetIdentity unable to find a MAC address match");
}
}
}


sub MCUpdateScripts($$$)
{
my $localpath = $_[0];
my $remoteshare = $_[1];
my $remotepath = $_[2];
# open the local directory, and create a hashtable
opendir LOCAL, $localpath;
@local_files = readdir LOCAL;
foreach $file(@local_files)
{
if($file =~ /.*\.pl$/) # we're only interested in .pl files
{
$local_files{$file} ="x";
#remember its the keys that matter
}
}
@local_files = ""; # be nice and conserve memory
closedir LOCAL;

`net use z: /delete`; # just to be sure
`net use z: $remoteshare`;
if(!$?) # if we got the share mapped successfully
{
opendir REMOTE, "z:$remotepath";
@remote_files = readdir REMOTE;
closedir REMOTE;
}
else
{
# returning a non-zero value means the update isnt successful
# If your circumstances require, you can use
# MCRaiseError here instead
return 1;
}

foreach $remote_file(@remote_files)
{
# check if matches a local file (hash key)
if($local_files{$remote_file} ne undef)
{
#the file exists, so do the version checking
print "Found a match on $remote_file\n";
#first get version number of local file
open FILE, "$localpath\\$remote_file";
<FILE>; $localversion = <FILE>;
$localversion =~ s/\$version=(\d+)/$1/;
close FILE;
# now get version number of remote file
open FILE, "z:$remotepath\\$remote_file";
<FILE>; $remoteversion = <FILE>;
$remoteversion =~ s/\$version=(\d+)/$1/;
close FILE;
if($remoteversion > $localversion)
{
# if the server version is newer, replace it
`xcopy z:\\$remotepath\\$remote_file $localpath\\$local_file`;
}
delete $local_files{$remote_file}; # delete hash entry
}
}

# every entry still in the hashtable has its file deleted
# (They werent on the server so must be obsolete
@deletables = keys(%local_files);
foreach $deletable(@deletables)
{
print "$deletable\n";
`del $localpath\\$deletable`;
}
# return a "success" code.
return 0;
}


sub MCConfigFromFile($)
{
open FILE, $_[0];
my $status = 0;
foreach $entry(<FILE>)
{
chomp($entry);
if(!($entry =~ /^[#;\s]/))
{
#its not a comment, so work out what to do with it
if($entry =~ /^\[/)
{
#its a section header, so strip the brackets
$entry =~ s/\[(.*)\]/$1/;
$status = $entry; # set the status
}
else
{
if($status eq "COMMANDS") #its a normal entry
{
`$entry`; #carry out the command,
#with error-checking
# If you don't want this to cause an error,
# rem out this line
if($?){MCRaiseError("Unable to execute $entry");}
}
else # it's a registry entry
{
my ($name, $value, $type_string) = 
split(/\s*,\s*/,$entry);
if($type_string) # its a value to add
{
# now set the type of entry to write
# The code only supports DWORD AND SZ 
# you may wish to change this
# by adding
# some else if's here!
if($type_string == REG_DWORD)
{$type = REG_DWORD;}
else{$type = REG_SZ;}
MCWriteRegistry($status,[$name,$value,$type]);
}
else # its a value to remove
{
MCDeleteValue($status, $entry);
}
}
}
}
}
close FILE;
}

BEGIN
{
#set up some generally useful (essential) variables.
$LogFileName = 'c:\ghostlog.txt';
$AdminMachine = 'myNTbox';
$AdminEmail = 'root@mymachine.mydomain.com';
$smtp_host = 'smtp.myorganisation.com';
$script_address = 'AutoScripter@myserver.myorganisation.com';
$logcount = 1;

# Write a log entry - any other useful startup code can be added here
open LOGFILE, ">> $LogFileName";
MCWriteLog ("MaintainAndConfigure module invoked");
close LOGFILE;
}


END
{
# Write a log entry - any other useful tidyup code can be added here
open LOGFILE, ">> $LogFileName";
MCWriteLog ("MaintainAndConfigure module released");
close LOGFILE;
}